export declare type Decoder<T> = (value: any) => Promise<T>;
export interface DecoderDict {
    [key: string]: Decoder<any>;
}
export declare type DecoderValueDict<T extends DecoderDict> = {
    [K in keyof T]: T[K] extends Decoder<infer U> ? U : never;
};
export declare class DecodeError extends Error {
    constructor(expectedType: string, receivedType: string);
}
export interface Composeable {
    <A, B>(decoder1: Decoder<A>, decoder2: Decoder<B>): Decoder<A | B>;
    <A, B, C>(decoder1: Decoder<A>, decoder2: Decoder<B>, decoder3: Decoder<C>): Decoder<A | B | C>;
    <A, B, C, D>(decoder1: Decoder<A>, decoder2: Decoder<B>, decoder3: Decoder<C>, decoder4: Decoder<D>): Decoder<A | B | C | D>;
    <A, B, C, D, E>(decoder1: Decoder<A>, decoder2: Decoder<B>, decoder3: Decoder<C>, decoder4: Decoder<D>, decoder5: Decoder<E>): Decoder<A | B | C | D | E>;
    <A, B, C, D, E, F>(decoder1: Decoder<A>, decoder2: Decoder<B>, decoder3: Decoder<C>, decoder4: Decoder<D>, decoder5: Decoder<E>, decoder6: Decoder<F>): Decoder<A | B | C | D | E | F>;
    <A, B, C, D, E, F, G>(decoder1: Decoder<A>, decoder2: Decoder<B>, decoder3: Decoder<C>, decoder4: Decoder<D>, decoder5: Decoder<E>, decoder6: Decoder<F>, decoder7: Decoder<G>): Decoder<A | B | C | D | E | F | G>;
    <A, B, C, D, E, F, G, H>(decoder1: Decoder<A>, decoder2: Decoder<B>, decoder3: Decoder<C>, decoder4: Decoder<D>, decoder5: Decoder<E>, decoder6: Decoder<F>, decoder7: Decoder<G>, decoder8: Decoder<H>): Decoder<A | B | C | D | E | F | G | H>;
    <A, B, C, D, E, F, G, H, I>(decoder1: Decoder<A>, decoder2: Decoder<B>, decoder3: Decoder<C>, decoder4: Decoder<D>, decoder5: Decoder<E>, decoder6: Decoder<F>, decoder7: Decoder<G>, decoder8: Decoder<H>, decoder9: Decoder<I>): Decoder<A | B | C | D | E | F | G | H | I>;
    <A, B, C, D, E, F, G, H, I, J>(decoder1: Decoder<A>, decoder2: Decoder<B>, decoder3: Decoder<C>, decoder4: Decoder<D>, decoder5: Decoder<E>, decoder6: Decoder<F>, decoder7: Decoder<G>, decoder8: Decoder<H>, decoder9: Decoder<I>, decoder10: Decoder<J>): Decoder<A | B | C | D | E | F | G | H | I | J>;
}
export interface Map {
    <A, B>(f: (value: A) => B, decoder: Decoder<A>): Decoder<B>;
    <A, B, C>(f: (value1: A, value2: B) => C, decoder1: Decoder<A>, decoder2: Decoder<B>): Decoder<C>;
    <A, B, C, D>(f: (value1: A, value2: B, value3: C) => D, decoder1: Decoder<A>, decoder2: Decoder<B>, decoder3: Decoder<C>): Decoder<D>;
    <A, B, C, D, E>(f: (value1: A, value2: B, value3: C, value4: D) => E, decoder1: Decoder<A>, decoder2: Decoder<B>, decoder3: Decoder<C>, decoder4: Decoder<D>): Decoder<E>;
    <A, B, C, D, E, F>(f: (value1: A, value2: B, value3: C, value4: D, value5: E) => F, decoder1: Decoder<A>, decoder2: Decoder<B>, decoder3: Decoder<C>, decoder4: Decoder<D>, decoder5: Decoder<E>): Decoder<F>;
    <A, B, C, D, E, F, G>(f: (value1: A, value2: B, value3: C, value4: D, value5: E, value6: F) => G, decoder1: Decoder<A>, decoder2: Decoder<B>, decoder3: Decoder<C>, decoder4: Decoder<D>, decoder5: Decoder<E>, decoder6: Decoder<F>): Decoder<G>;
    <A, B, C, D, E, F, G, H>(f: (value1: A, value2: B, value3: C, value4: D, value5: E, value6: F, value7: G) => H, decoder1: Decoder<A>, decoder2: Decoder<B>, decoder3: Decoder<C>, decoder4: Decoder<D>, decoder5: Decoder<E>, decoder6: Decoder<F>, decoder7: Decoder<G>): Decoder<H>;
    <A, B, C, D, E, F, G, H, I>(f: (value1: A, value2: B, value3: C, value4: D, value5: E, value6: F, value7: G, value8: H) => I, decoder1: Decoder<A>, decoder2: Decoder<B>, decoder3: Decoder<C>, decoder4: Decoder<D>, decoder5: Decoder<E>, decoder6: Decoder<F>, decoder7: Decoder<G>, decoder8: Decoder<H>): Decoder<I>;
    <A, B, C, D, E, F, G, H, I, J>(f: (value1: A, value2: B, value3: C, value4: D, value5: E, value6: F, value7: G, value8: H, value9: I) => J, decoder1: Decoder<A>, decoder2: Decoder<B>, decoder3: Decoder<C>, decoder4: Decoder<D>, decoder5: Decoder<E>, decoder6: Decoder<F>, decoder7: Decoder<G>, decoder8: Decoder<H>, decoder9: Decoder<I>): Decoder<J>;
    <A, B, C, D, E, F, G, H, I, J, K>(f: (value1: A, value2: B, value3: C, value4: D, value5: E, value6: F, value7: G, value8: H, value9: I, value10: J) => K, decoder1: Decoder<A>, decoder2: Decoder<B>, decoder3: Decoder<C>, decoder4: Decoder<D>, decoder5: Decoder<E>, decoder6: Decoder<F>, decoder7: Decoder<G>, decoder8: Decoder<H>, decoder9: Decoder<I>, decoder10: Decoder<J>): Decoder<K>;
}
export declare const decodeString: <T>(decoder: Decoder<T>) => (input: string) => Promise<T>;
export declare const decodeValue: <T>(decoder: Decoder<T>) => (input: any) => Promise<T>;
export declare const simpleDecoder: <T>(type: string, f: (value: any) => boolean) => (value: any) => Promise<T>;
export declare const str: Decoder<string>;
export declare const num: Decoder<number>;
export declare const bool: Decoder<boolean>;
export declare const nil: Decoder<null>;
export declare const array: <T>(decoder: Decoder<T>) => Decoder<T[]>;
export declare const oneOf: Composeable;
export declare const nullable: <T>(decoder: Decoder<T>) => Decoder<T | null>;
export declare const maybe: <T>(decoder: Decoder<T>) => Decoder<T | undefined>;
export declare const object: <T extends DecoderDict>(decoders: T) => Decoder<DecoderValueDict<T>>;
export declare const field: <T>(name: string, decoder: Decoder<T>) => Decoder<T>;
export declare const compose: Composeable;
export declare const map: Map;
